/*-------------------------------------------------------------------------+
| Copyright 2023 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.ext.reuse.service;

import static org.eclipse.emf.ecore.util.EcoreUtil.replace;
import static org.fortiss.tooling.ext.variability.util.VariabilityUtils.getOptVarPointSpecification;
import static org.fortiss.tooling.kernel.utils.EcoreUtils.replaceEObjectReferences;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.emf.ecore.EObject;
import org.fortiss.tooling.base.model.element.IHierarchicElement;
import org.fortiss.tooling.base.model.element.IHierarchicElementContainer;
import org.fortiss.tooling.base.model.element.IModelElement;
import org.fortiss.tooling.ext.variability.model.OptionalVariationPointSpecification;
import org.fortiss.tooling.kernel.model.INamedCommentedElement;
import org.fortiss.tooling.kernel.model.INamedElement;

/***
 * Implemented basic {@link IReuseProvider} based on {@link EObject}. Can be
 * extended or overridden by more specific implemented {@link IReuseProvider}.
 *
 * @author bergemann
 *
 * @param <T>
 *            The subclass of {@link EObject} for which this provider base shall
 *            be extended.
 */
public class ReuseProviderBase<T extends EObject> implements IReuseProvider<T> {

	/** {@inheritDoc} */
	@Override
	public boolean prepareReuseElementForLibrary(T reuseElement) {
		if(reuseElement instanceof IModelElement) {
			removeVariabilitySpecifications((IModelElement)reuseElement);
		}
		return true;
	}

	/** {@inheritDoc} */
	@Override
	public boolean updateReuseElement(T elementToBeUpdated, T newElement) {
		replaceEObjectReferences(elementToBeUpdated, newElement);
		replace(elementToBeUpdated, newElement);
		return true;
	}

	/** {@inheritDoc} */
	@Override
	public boolean setReuseElementName(T reuseElement, String name) {
		if(reuseElement instanceof INamedElement) {
			((INamedElement)reuseElement).setName(name);
			return true;
		}
		return false;
	}

	/** {@inheritDoc} */
	@Override
	public String getReuseElementName(T reuseElement) {
		if(reuseElement instanceof INamedElement) {
			return ((INamedElement)reuseElement).getName();
		}
		return null;
	}

	/** {@inheritDoc} */
	@Override
	public boolean setReuseElementComment(T reuseElement, String comment) {
		if(reuseElement instanceof INamedCommentedElement) {
			((INamedCommentedElement)reuseElement).setComment(comment);
			return true;
		}
		return false;
	}

	/** {@inheritDoc} */
	@Override
	public String getReuseElementComment(T reuseElement) {
		if(reuseElement instanceof INamedCommentedElement) {
			return ((INamedCommentedElement)reuseElement).getComment();
		}
		return null;
	}

	/** {@inheritDoc} */
	@Override
	public List<Class<? extends EObject>> getPossibleExternalReferenceClasses() {
		// Can only be specified for specific elements (if wanted) --> here only empty.
		return new ArrayList<Class<? extends EObject>>();
	}

	/**
	 * Removes unnecessary variability specifications from the given element and from all of its
	 * contained elements. The method is recursion-based!
	 */
	private void removeVariabilitySpecifications(IModelElement nextElement) {
		// Removal.
		OptionalVariationPointSpecification unnecessarySpec =
				getOptVarPointSpecification(nextElement);
		if(unnecessarySpec != null) {
			nextElement.getSpecifications().remove(unnecessarySpec);
		}

		// Recursion.
		if(nextElement instanceof IHierarchicElementContainer) {
			for(IHierarchicElement elem : ((IHierarchicElementContainer)nextElement)
					.getContainedElements()) {
				removeVariabilitySpecifications(elem);
			}
		}
	}
}
